/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource['ibm.portal.search.widgets.DisplayResultsWidgets']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm.portal.search.widgets.DisplayResultsWidgets'] = true;
dojo.require('dojo.string');
dojo.require('dojo.date.locale');
dojo.require('dojo.date.stamp');
dojo.require('dojo.i18n');
dojo.require('dojo.number');
dojo.require('dijit._Widget');
dojo.require('dijit._Container');
dojo.require('ibm.portal.search.widgets.common');
dojo.require("com.ibm.portal.xslt");
dojo.requireLocalization('dojo.cldr', 'gregorian', djConfig.locale, "");
dojo.requireLocalization('dojo.cldr', 'number', djConfig.locale, "");
dojo.provide('ibm.portal.search.widgets.DisplayResultsWidgets');

dojo.declare(
	'ibm.portal.search.widgets.SearchService', [dijit._Widget, dijit._Container],
	{
	//summary:
	//	a wrapper for DisplayResultsWidget and ResultsFetcher
	
	//topics for intercommunication between child widgets
	nextPageTopic: null,
	prevPageTopic: null,
	firstPageTopic: null,
	itemsPerPageTopic: null,
	displayResultsTopic: null,
	newQueryTopic: null,
	
	//summary:
	//	create the topics used by child widgets
	//	invoke the startup() method on child widgets
	startup: function() {
		this.newQueryTopic = this.id + '_NEW_QUERY_TOPIC';
		this.nextPageTopic = this.id + '_NEXT_PAGE_TOPIC';
		this.prevPageTopic = this.id + '_PREV_PAGE_TOPIC';
		this.firstPageTopic = this.id + '_FIRST_PAGE_TOPIC';
		this.itemsPerPageTopic = this.id + '_ITEMS_PER_PAGE_TOPIC';
		this.displayResultsTopic = this.id + '_DISPLAY_RESULTS_TOPIC';
		
		this.getChildren().forEach(function(child) {child.startup()});
	}
});

dojo.declare(
	'ibm.portal.search.widgets.ResultsFetcher', [dijit._Widget, dijit._Contained],
	{
	//summary:
	//	Retrieves search results from the server

	//numItems: Number
	//	the number of search results to display in each page
	numItems: 10,
	
	//currentPage: Number
	//	the index of the current page view of the search results
	currentPage: 0,
	
	//query: ibm.portal.search.Query
	//	the query object
	query: null,
	
	//helpUrl: String
	//	The url of the search help
	helpUrl: null,
	
	//resourceBundle: Object
	//	contains key/value pairs of translated strings
	resourceBundle: null,
	
	errorMessageTemplate: '<div class="wpsStatusMsg"><br/><table><tbody><tr><td><img width="38" height="38" border="0" align="absmiddle" src="${errorIcon}" alt="${error}"/></td><td><span class="wpsFieldWarningText">${errorMessage}</span><br/><span class="wpsFieldWarningText">${detailedMessage}</span></td></tr></tbody></table><br/></div>',

	suggestionsTemplate: '<br/><br/>${suggestionsHeader}<ul type="disc"><li>${suggestion1}</li><li>${suggestion2}</li><li>${suggestion3}</li></ul>',
	
	helpLinkTemplate: '<a class="helpLink" href="javascript:void(0);" onclick="ibm.portal.search.util.openSideWindow(\'${0}\'); return false;">${1}</a>',
	
	errorIcon: ibm.portal.search.util.getContextRoot() + '/images/icons/error.gif',
	
	searchFeedUrl: null,
		
	//override dijit._Widget's method
	startup: function () {
		if (djConfig.isDebug) {
			console.debug('ResultsFetcher: startup');
		}
		var parentWidget = this.getParent();
		dojo.subscribe(ibm.portal.search.NEW_QUERY_TOPIC, this, 'newQuery');
		dojo.subscribe(parentWidget.nextPageTopic, this, 'nextPage');
		dojo.subscribe(parentWidget.prevPageTopic, this, 'prevPage');
		dojo.subscribe(parentWidget.firstPageTopic, this, 'firstPage');
		dojo.subscribe(parentWidget.itemsPerPageTopic, this, 'itemsPerPage');
		dojo.addClass(this.domNode, 'resultsSummary');
	},
	
	//Summary:
	//	handle a new query submitted by the user
	//newQuery: ibm.portal.search.Query
	//	the query that was submitted
	newQuery: function(/*ibm.portal.search.Query*/ newQuery) {
		if (djConfig.isDebug) {
			console.debug('ResultsFetcher: newQuery');
		}
		this.currentPage = 0;
		this.query = newQuery;
		this.fetchResults(true);
	},
	//Summary:
	//	fetch search results from the server
	fetchResults: function(/*boolean*/displaySummaryMessage) {
		dojo.fadeOut({node: this.domNode, duration: 100}).play();
		var _this = this;
		return dojo.xhrGet({
			url: this.createServiceUrl(),
			load: function(response, ioArgs) {
				if (djConfig.isDebug) {
					console.debug(_this.id + 'xhrGet load');
				}
				var newResultSet = ibm.portal.xml.xslt.loadXmlString(response);//required because of IE issues
				dojo.fadeIn({node: _this.domNode, duration: 300}).play();
				if (displaySummaryMessage) {
					_this.displayTotalResultsMessage(newResultSet);
				}
				dojo.publish(_this.getParent().displayResultsTopic, [newResultSet]);
			},
			error: function(response, ioArgs) {
				if (djConfig.isDebug) {
					console.debug(_this.id + 'xhrGet error');
				}
				var responseXml = ibm.portal.xml.xslt.loadXmlString(ioArgs.xhr.responseText);//required because of IE issues
				var responseMessage = ibm.portal.search.util.getXpathValue(responseXml, '/error/message');
				dojo.fadeIn({node: _this.domNode, duration: 300}).play();
				if (djConfig.isDebug) {
					console.debug(responseMessage);
				}
				_this.displayErrorMessage(responseMessage);
			},
			handleAs: 'text'
		});
	},
	//Summary:
	//	create the URL for the search REST servlet, based on query parameters
	createServiceUrl: function() {
		return ibm.portal.search.util.getServiceUrl(this.searchFeedUrl, 'search', {
			query: this.query.getFullQueryText(),
			scope: this.query.scope.getValue(),
			queryLang: this.query.language,
			locale: djConfig.locale,
			results: this.numItems,
			start: this.currentPage * this.numItems
		});
	},
	//Summary:
	//	respond to a 'next_page' event
	nextPage: function() {
		this.currentPage++;
		this.fetchResults(true);
	},
	//Summary:
	//	respond to a 'prev_page' event
	prevPage: function() {
		this.currentPage--;
		this.fetchResults(true);
	},
	//Summary:
	//	respond to a 'first_page' event
	firstPage: function() {
		this.currentPage = 0;
		this.fetchResults(true);
	},
	//Summary:
	//	respond to a 'items_per_page' event
	itemsPerPage: function(/*Number*/ newNumItems) {
		this.numItems = newNumItems;
		this.currentPage = 0;
		this.fetchResults(true);
	},
	//summary:
	// display total results message, like "93 results found in All Sources for 'test'"
	displayTotalResultsMessage: function(/*XmlDoc*/ resultSet) {	
		var totalResults = new Number(ibm.portal.search.util.getXpathValue(resultSet, '/atom:feed/opensearch:totalResults'));
		var exact = (ibm.portal.search.util.getXpathValue(resultSet, '/atom:feed/opensearch:totalResults/@exact') == "true");
		var itemsPerPage = new Number(ibm.portal.search.util.getXpathValue(resultSet, '/atom:feed/opensearch:itemsPerPage'));
		var startIndex = new Number(ibm.portal.search.util.getXpathValue(resultSet, '/atom:feed/opensearch:startIndex'));
		
		var scopeName = this.emphasize(this.query.scope.label);
		var queryText = this.query.getFullQueryText();
		
		queryText = this.emphasize(queryText);
		var messageTemplate = null;
		if (!exact && totalResults > startIndex + itemsPerPage) {
			messageTemplate = this.resourceBundle.resultsSummaryMore;
			totalResults = startIndex + itemsPerPage;
		} else {
			messageTemplate = this.resourceBundle[(totalResults == 1)?
				'resultsSummary1': 'resultsSummary'];
		}		
		
		var numOfResults = this.emphasize(dojo.number.format(totalResults, {locale: djConfig.locale}));
		var message = dojo.string.substitute(messageTemplate, 
			[numOfResults, scopeName, queryText]);
		var suggestionsMessage = '';
		if (totalResults == 0) {
			var helpLink = dojo.string.substitute(this.helpLinkTemplate, 
				[this.helpUrl, this.resourceBundle.helpLink]);
			var suggestion3 = dojo.string.substitute(this.resourceBundle.suggestion3, [helpLink]);
			suggestionsMessage = dojo.string.substitute(this.suggestionsTemplate,
				{
					suggestionsHeader: this.resourceBundle.suggestionsHeader,
					suggestion1: this.resourceBundle.suggestion1,
					suggestion2: this.resourceBundle.suggestion2,
					suggestion3: suggestion3
				});
		}
		this.domNode.innerHTML = message + suggestionsMessage;
	},
	//summary:
	//	return the text surrounded by a <strong> tag
	emphasize: function(text) {
		return '<strong>' + text + '</strong>';
	},
	//Summary:
	//	display error message when fetching results from the server failed
	displayErrorMessage: function(detailedErrorMsg) {
		if (djConfig.isDebug) {
			console.debug('displayErrorMessage');
		}
		dojo.fadeIn({node: this.domNode, duration: 100}).play();
		var errorMessage = dojo.string.substitute(this.errorMessageTemplate, {
			error: this.resourceBundle.error,
			errorIcon: this.errorIcon,
			errorMessage: this.resourceBundle.genericErrorMsg,
			detailedMessage: ""
		})
		if (djConfig.isDebug) {
			console.debug(errorMessage);
		}
		this.domNode.innerHTML = errorMessage;
	}
});

dojo.declare(
	'ibm.portal.search.widgets.SponsoredLinksFetcher', ibm.portal.search.widgets.ResultsFetcher,
	{
	//summary:
	//	Retrieves search results from a sponsored links collection
	
	//collectionId: String
	//	the id of the search collection which is searched
	collectionId: null,
	
	//query: String
	//	the query to search for
	query: null,

	//searchInContent: Boolean
	//	determines whether to perform full-text search, or just keyword search
	searchInContent: false,

	//override dijit._Widget's method
	startup: function () {
		if (djConfig.isDebug) {
			console.debug(this.id + ': startup');
		}
		var parentWidget = this.getParent();
		dojo.subscribe(parentWidget.newQueryTopic, this, 'newQuery');
	},
	
	//summary:
	//	react to a newQuery event
	newQuery: function (query) {
		this.query = query;
		this.fetchResults();
	},
	
	//Summary:
	//	create the URL for the search REST servlet, based on query parameters
	createServiceUrl: function() {
		return ibm.portal.search.util.getServiceUrl(this.searchFeedUrl, 'search', {
			query: (this.searchInContent? '': 'keywords:') + this.query,
			index: this.collectionId,
			locale: djConfig.locale,
			results: this.numItems
		});
	}
});

dojo.declare(
	'ibm.portal.search.widgets.ExternalFeedFetcher', ibm.portal.search.widgets.SponsoredLinksFetcher,
	{
	//summary:
	//	Retrieves search results from an external search engine through the proxy servlet
	
	//searchEngineUrl: String
	//	the template URL of the external search engine
	searchEngineUrl: null,
	
	proxyUrl: null,
	
	//Summary:
	//	create the URL for the search REST servlet, based on query parameters
	createServiceUrl: function() {
		var urlBuffer = [this.proxyUrl];
		this.searchEngineUrl = this.searchEngineUrl.replace(/:\/\//, "/");
		urlBuffer.push(dojo.string.substitute(this.searchEngineUrl, {searchTerms: this.query}));
		return urlBuffer.join('');
	},
	
	//Summary:
	//	display error message when fetching results from the external site failed
	displayErrorMessage: function(detailedErrorMsg) {
		var start = this.searchEngineUrl.indexOf("/") + 1;
		var end = this.searchEngineUrl.indexOf("?");
		var host = this.searchEngineUrl.substring(start, end);
		
		dojo.style(this.domNode, "opacity", 1);
		var errorMessage = dojo.string.substitute(this.errorMessageTemplate, {
			error: this.resourceBundle.error,
			errorIcon: this.errorIcon,
			errorMessage: dojo.string.substitute(this.resourceBundle.externalErrorMsg, [this.emphasize(host)]),
			detailedMessage: ""
		})
		this.domNode.innerHTML = errorMessage;
	}
});

dojo.declare(
	'ibm.portal.search.widgets.ResultsDisplayWidget', [dijit._Widget, dijit._Contained],
	{
	//summary:
	//	Displays search results
	
	//xsltUrl: String
	//	the URL of the XSLT which trasforms results feed to html
	xsltUrl: dojo.moduleUrl('ibm.portal.search.xslt','results.xsl'),
	
	//resourceBundle: Object
	//	contains key/value pairs of translated strings
	resourceBundle: null,
	
	//override dijit._Widget's method
	startup: function () {
		if (djConfig.isDebug) {
			console.debug(this.id + ': startup');
		}
		dojo.subscribe(this.getParent().displayResultsTopic, this, 'displayResults');		
	},
	//Summary:
	//	returns the path of the XSLT file to transform the results to HTML
	getXsltPath: function(/*DomNode*/ xmlDoc) {
		return this.xsltUrl.toString();
	},
	//Summary:
	//	creates a map of all XSLT parameters
	getXsltParams: function() {
		this.resourceBundle["contextRoot"] = ibm.portal.search.util.getContextRoot();
		if (!this.isLeftToRight()) {
			this.resourceBundle["isRTL"] = "_rtl";
		}
		return this.resourceBundle;
	},	
	//Summary:
	//	display search results
	displayResults: function(resultsDoc) {
		var xsltPath = this.getXsltPath(resultsDoc);
		var xslTransformer = com.ibm.portal.xslt.TRANSFORMER_FACTORY.newTransformer(xsltPath);
				
		xslTransformer.transformToRegion(resultsDoc, this.getXsltParams(), this.domNode, document);
		
		this.postFix(resultsDoc);
		dojo.fadeIn({node: this.domNode, duration: 300}).play();				
	},
	//summary:
	//	fade out the results view, when a new query is submitted
	fadeOut: function() {
		dojo.fadeOut({node: this.domNode, duration: 100}).play();
	},
	//summary:
	//	fix pagination links by assigning them an 'onclick' method
	fixPaginationLink: function(/*String*/ linkClass, /*String*/ topic) {
		var _this = this;
		dojo.query(linkClass, this.domNode).forEach(function(linkNode) {
	        linkNode.onclick = function () {
				_this.fadeOut();
				dojo.publish(_this.getParent()[topic], []);
			};
		});
	},
	//summary:
	//	fix the html created by XSL transform
	postFix: function (/*DomNode*/ resultsDoc) {
		var _this = this;
		//fix date strings
		dojo.query('.date', this.domNode).forEach(function(dateNode) {
	        dateNode.innerHTML = _this.formatDate(dateNode.innerHTML);
		});

		//fix pagination links		
		this.fixPaginationLink('.firstPageLink', 'firstPageTopic');
		this.fixPaginationLink('.prevPageLink', 'prevPageTopic');
		this.fixPaginationLink('.nextPageLink', 'nextPageTopic');
		
		dojo.query('.itemsPerPageLink', this.domNode).forEach(function(linkNode) {
			var newItemsPerPage = new Number(linkNode.firstChild.data);
			
			linkNode.onclick = function () {
				if (djConfig.isDebug) {
					console.debug('newItemsPerPage: ' + newItemsPerPage);
				}
				_this.fadeOut();
				dojo.publish(_this.getParent().itemsPerPageTopic, [newItemsPerPage]);
			};
		});
		//calculate end index
		var startIndex = new Number(ibm.portal.search.util.getXpathValue(resultsDoc, '/atom:feed/opensearch:startIndex'));
		var itemsPerPage = new Number(ibm.portal.search.util.getXpathValue(resultsDoc, '/atom:feed/opensearch:itemsPerPage'));
		var totalResults = new Number(ibm.portal.search.util.getXpathValue(resultsDoc, '/atom:feed/opensearch:totalResults'));
		var endIndex = Math.min(totalResults, startIndex + itemsPerPage);
	   
		//update showing results message
		dojo.query('.showingResultsMessage', this.domNode).forEach(function(message) {
	   		var messageTemplate = _this.resourceBundle.showingResultsMessage;
			message.innerHTML = dojo.string.substitute(messageTemplate, 
				[dojo.number.format(startIndex + 1, {locale: djConfig.locale}), 
					dojo.number.format(endIndex, {locale: djConfig.locale})]);
		});
		
		//fix formatted text escaping problem
		this.unescapeSummary();
		
		//fix IE issues with empty cells
		dojo.query('td, span', this.domNode).forEach(function(cell) {
			if (cell.innerHTML == '') {
				cell.appendChild(document.createTextNode(" "));
			}
		});
		
		//call semantic tag service
		if (typeof SemTagSvc != "undefined") {
			SemTagSvc.parseDom(null, this.domNode);
		}
	},
	//summary:
	//	unescape html tags inside a result's summary
	unescapeSummary: function () {
		dojo.query('.preFormatted', this.domNode).forEach(function(element) {
			if (element.childNodes.length == 0) {
				return;
			}
			var formattedText = element.innerHTML;
			formattedText = formattedText.replace(/&lt;/g, '<').replace(/&gt;/g, '>').replace(/&quot;/g, '"').replace(/&amp;/g, '&');
			element.innerHTML = formattedText;
		});
	},
	//summary:
	//	parse a date string retrived from the server and return the formatted string in the right locale
	formatDate: function(dateString)  {
		var date = dojo.date.stamp.fromISOString(dateString);
		return dojo.date.locale.format(date, {locale: djConfig.locale});
	}
});

dojo.declare(
	'ibm.portal.search.widgets.SimpleResultsDisplayWidget', ibm.portal.search.widgets.ResultsDisplayWidget,
	{
	//summary:
	//	Displays search results
	
	//xsltUrl: String
	//	the URL of the XSLT which trasforms results feed to html
	xsltUrl: dojo.moduleUrl('ibm.portal.search.xslt','simpleResults.xsl'),
	
	//Summary:
	//	creates a map of all XSLT parameters
	getXsltParams: function() {
		if (djConfig.isDebug) {
			console.debug('SimpleResultsDisplayWidget: getXsltParams');
		}
		return null;
	},
	
	//summary:
	//	fix the html created by XSL transform
	postFix: function (/*DomNode*/ resultsDoc) {
		//fix formatted text escaping problem
		this.unescapeSummary();
	}
});

dojo.declare(
	'ibm.portal.search.widgets.ExternalFeedDisplayWidget', ibm.portal.search.widgets.SimpleResultsDisplayWidget,
	{
	//summary:
	//	Displays search results
	
	//xsltMap: Object
	//	A map between all supported syndication formats and the matching XSLT files
	xsltMap: {
		'rss2.0': 'rss20Results.xsl',
		'feed0.3': 'atom03Results.xsl',
		'feed': 'atom10Results.xsl'
	},
	
	//externalXsltUrl: String
	//	the URL of an XSLT which trasforms results to atom feed
	externalXsltUrl: null,
	
	//searchEngineFullPageUrl: String
	//	the URL template of a the search engine's main html page
	searchEngineFullPageUrl: null,
	
	//numOfEntries: Number
	//	the number of search results to display
	numOfEntries: 3,
	
	//query: String
	//	the query to search for in full page mode
	query: null,
	
	//override dijit._Widget's method
	startup: function () {
		dojo.subscribe(this.getParent().newQueryTopic, this, 'newQuery');
		this.inherited(arguments);
	},
	
	//summary:
	//	react to a newQuery event
	newQuery: function (query) {
		this.query = query;
	},
	
	//Summary:
	//	returns the path of the XSLT file to transform the results to HTML
	getXsltPath: function(/*DomNode*/ xmlDoc) {
		if (djConfig.isDebug) {
			console.debug('ExternalFeedDisplayWidget: getXsltPath');
		}
		
		if (this.externalXsltUrl != null && this.externalXsltUrl.length > 0) {
			return this.externalXsltUrl;
		}
		
		var feedName = xmlDoc.documentElement.tagName;
		var version = xmlDoc.documentElement.getAttribute('version');
		if (version != null) {
			feedName += version;
		}
		var xslName = this.xsltMap[feedName];
		if (xslName == null) {
			throw "unsupported syndication format: " + feedName;
		}
		this.xsltUrl = dojo.moduleUrl('ibm.portal.search.xslt', xslName);
		return this.inherited(arguments);//call parent method, like 'super'
	},
	
	//Summary:
	//	creates a map of all XSLT parameters
	getXsltParams: function() {
		return  {"numOfEntries": this.numOfEntries};
	},
	
	//summary:
	//	fix the html created by XSL transform
	postFix: function (/*DomNode*/ resultsDoc) {
		//call parent's method
		this.inherited(arguments);
		
		if (this.searchEngineFullPageUrl == null || this.query == null) {
			return;
		}
		
		this.searchEngineFullPageUrl = dojo.string.substitute(this.searchEngineFullPageUrl, 
			{searchTerms: this.query});
		var moreLinkContainer = document.createElement('h3');
		moreLinkContainer.innerHTML = dojo.string.substitute('<a target="_blank" href="${0}">${1}</a>', 
			[this.searchEngineFullPageUrl, this.resourceBundle.moreResults]);
		this.domNode.appendChild(moreLinkContainer);
	}
});

}
